
#include "Arduino.h"
#include "math.h"
#include <LiquidCrystal.h>
#include "sht.h"

#define CBM_RUN_TIME    60
#define CBM_LCD_ROWS    2
#define CBM_MAX_ERRORS  3
#define CBM_POWER_PIN   10

#define SHT_CLOCK_PIN 9
#define SHT_DATA_PIN  8
sht15_sensor sht(SHT_CLOCK_PIN, SHT_DATA_PIN);

LiquidCrystal lcd(12, 11, 5, 4, 3, 2);


float c_to_f(float c) {
  return (c * 9 / 5) + 32; 
}


void lcd_display_data(float temp_f, float rh_true, float cloud_ft, float dew_f) {

  lcd.clear();
  
  lcd.setCursor(0, 0);
  lcd.print("T: ");
  lcd.print(round(temp_f));
  lcd.print((char)223);
  lcd.print("F");

  lcd.setCursor(9, 0);
  lcd.print("RH:");
  if (round(rh_true) < 100) lcd.print(" ");
  lcd.print(round(rh_true));
  lcd.print("%");

  lcd.setCursor(0, 1);
  lcd.print("Clouds ");
//lcd.print("D: ");
//lcd.print(round(dew_f));
//lcd.print("F ");
  
  if (cloud_ft < 100) {
      lcd.print("0' (fog?)");
  } else if (cloud_ft > 9999) {
    lcd.print(round(cloud_ft/1000)*1000);
    lcd.print("'");
  } else {
    lcd.print(round(cloud_ft/100)*100);
    lcd.print("' AGL");
  }
  

}


void power_down(void) {
  digitalWrite(CBM_POWER_PIN, HIGH);
  digitalWrite(CBM_POWER_PIN, HIGH);
  digitalWrite(CBM_POWER_PIN, HIGH);
}


void handle_error(char *s, int fatal) {
  static int err_count = 0;
  static char err_str[64];

  if (err_count == 0) strncpy(err_str, s, 63);
  err_count++;

  if (!fatal && err_count < CBM_MAX_ERRORS)
    return;

  lcd.clear();
  lcd.print("ERROR:");
  lcd.print(err_count);
  lcd.setCursor(0, 1);
  lcd.print(err_str);

  delay(7000);
  power_down();
}


void setup() {
//  Serial.begin(9600);
  pinMode(CBM_POWER_PIN, OUTPUT);
  digitalWrite(CBM_POWER_PIN, LOW);
  lcd.begin(16, 2);
  lcd.clear();
}


void loop() {
  //these can take a bit to get the values (100ms or so)
  float so_t, temp_c, temp_f;
  float so_rh, rh_linear, rh_true;
  float dew_c, dew_f;
  float cloud_ft;


  if (millis() / 1000 > CBM_RUN_TIME) {
       lcd.clear();
       lcd.print("Goodbye...");
       delay(1000);
       power_down();
       return;
  }
  
  delay(1000);

  if (!sht.read_temperature(&so_t, &temp_c))
    return handle_error(sht.get_error(), false);

  if (!sht.read_humidity(&so_rh, &rh_linear))
    return handle_error(sht.get_error(), false);
    
  if (temp_c < -20 || temp_c > 50)
    return handle_error("Bad Temperature", true);

  if (rh_linear < 0 || rh_linear > 101)
    return handle_error("Bad Humidity", true);

  rh_true = sht.rh_compensate(temp_c, so_rh, rh_linear);
  dew_c = sht.dew_point(temp_c, rh_true);

  temp_f = c_to_f(temp_c);
  dew_f  = c_to_f(dew_c);

  cloud_ft = (temp_f - dew_f) / 4.4 * 1000;

  lcd_display_data(temp_f, rh_true, cloud_ft, dew_f);
  
}




